
module int_cont #(
    parameter           WIDTH = 8
)(
    input                   reset_n,    // reset
    input                   clk,        // clock
    input       [1:0]       addr,       // internal address
    input                   sel,        // internal select
    input                   write,      // internal write enable
    input       [31:0]      wdata,      // internal write data
    output      [31:0]      rdata,      // internal read data
    input       [WIDTH-1:0] int_in,     // int input
    output                  irq7,       // irq output
    output                  irq6,       // irq output
    output                  irq5,       // irq output
    output                  irq4,       // irq output
    output                  irq3,       // irq output
    output                  irq2,       // irq output
    output                  irq1,       // irq output
    output                  irq0        // irq output
);

//-------------- internal signals --------------
    reg          [WIDTH-1:0] rdata_mux;
    reg          [WIDTH-1:0] int_ena;
    reg          [WIDTH-1:0] int_sta;
    reg          [WIDTH-1:0] int_mode;
    reg          [WIDTH-1:0] int_pol;
    wire                     int_sta_sel;
    //reg          [WIDTH-1:0] int_sta_reset;

    reg          [WIDTH-1:0] int_d1;
    reg          [WIDTH-1:0] int_d2;
    wire         [WIDTH-1:0] int_pos;
    wire         [WIDTH-1:0] int_neg;

//-------------- logic description --------------

//-------------- register --------------
    always @(posedge clk or negedge reset_n) begin
        if(!reset_n) begin
            int_ena <= 0;
            int_mode <= 0;
            int_pol <= 0;
        end
        else if(sel && write) begin
            case(addr)
                0 : int_ena  <= wdata[WIDTH-1:0];
                2 : int_mode <= wdata[WIDTH-1:0];
                3 : int_pol  <= wdata[WIDTH-1:0];
            endcase
        end
    end

    assign int_sta_sel = (sel && write && (addr == 1));
    
//-------------- read data --------------
    assign rdata = {{(32-WIDTH){1'b0}}, rdata_mux};

    always @(posedge clk or negedge reset_n) begin
        if(!reset_n) begin
            rdata_mux <= 0;
        end
        else begin
            case(addr)
                0 : rdata_mux <= int_ena;
                1 : rdata_mux <= int_sta;
                2 : rdata_mux <= int_mode;
                3 : rdata_mux <= int_pol;
                default: rdata_mux <= 0;
            endcase
        end
    end

//-------------- irq --------------
 
  generate
  genvar i;
  for(i=0; i<WIDTH; i=i+1) begin : gen_int_sta
  
    always @(posedge clk or negedge reset_n) begin
        if(!reset_n) begin
            int_sta[i] <= 1'b0;
        end
        else if(int_mode[i]) begin
            if(int_sta[i]) begin
                if(int_sta_sel && !wdata[i]) begin
                    int_sta[i] <= 1'b0;
                end
            end
            else begin
                if(int_pol[i] && int_pos[i]) begin    // "H"edge
                    int_sta[i] <= 1'b1;
                end
                else if(!int_pol[i] && int_neg[i]) begin  // "L"edge
                    int_sta[i] <= 1'b1 ;
                end
            end
        end
        else begin    // level
            int_sta[i] <= (int_pol[i] == int_d2[i]);
        end
    end
    
    always @(posedge clk or negedge reset_n) begin
        if(!reset_n) begin
            int_d1[i] <= 1'b0;
            int_d2[i] <= 1'b0;
        end
        else begin
            int_d1[i] <= int_in[i];
            int_d2[i] <= int_d1[i];
        end
    end

    assign int_pos[i] = ~int_d2[i] &  int_d1[i];
    assign int_neg[i] =  int_d2[i] & ~int_d1[i];
  end
  endgenerate

//-------------- irq --------------
    assign irq7 = int_ena[7] & int_sta[7];
    assign irq6 = int_ena[6] & int_sta[6];
    assign irq5 = int_ena[5] & int_sta[5];
    assign irq4 = int_ena[4] & int_sta[4];
    assign irq3 = int_ena[3] & int_sta[3];
    assign irq2 = int_ena[2] & int_sta[2];
    assign irq1 = int_ena[1] & int_sta[1];
    assign irq0 = int_ena[0] & int_sta[0];

endmodule
